@@ -1,105 +1,5 @@
 open Import
 
-module Source = struct
-  type fetch =
-    { url : Loc.t * string
-    ; checksum : (Loc.t * Checksum.t) option
-    }
-
-  type t =
-    | External_copy of Loc.t * Path.External.t
-    | Fetch of fetch
-
-  let remove_locs = function
-    | External_copy (_loc, path) -> External_copy (Loc.none, path)
-    | Fetch { url = _loc, url; checksum } ->
-      Fetch
-        { url = Loc.none, url
-        ; checksum = Option.map checksum ~f:(fun (_loc, checksum) -> Loc.none, checksum)
-        }
-  ;;
-
-  let equal a b =
-    match a, b with
-    | External_copy (loc, path), External_copy (other_loc, other_path) ->
-      Loc.equal loc other_loc && Path.External.equal path other_path
-    | ( Fetch { url = loc, url; checksum }
-      , Fetch { url = other_loc, other_url; checksum = other_checksum } ) ->
-      Loc.equal loc other_loc
-      && String.equal url other_url
-      && Option.equal
-           (fun (loc, checksum) (other_loc, other_checksum) ->
-             Loc.equal loc other_loc && Checksum.equal checksum other_checksum)
-           checksum
-           other_checksum
-    | _ -> false
-  ;;
-
-  let to_dyn = function
-    | External_copy (_loc, path) ->
-      Dyn.variant "External_copy" [ Path.External.to_dyn path ]
-    | Fetch { url = _loc, url; checksum } ->
-      Dyn.variant
-        "Fetch"
-        [ Dyn.string url
-        ; Dyn.option (fun (_loc, checksum) -> Checksum.to_dyn checksum) checksum
-        ]
-  ;;
-
-  module Fields = struct
-    let copy = "copy"
-    let fetch = "fetch"
-    let url = "url"
-    let checksum = "checksum"
-  end
-
-  let decode_fetch =
-    let open Decoder in
-    let+ url = field Fields.url (located string)
-    and+ checksum = field_o Fields.checksum (located string) in
-    let checksum =
-      match checksum with
-      | None -> None
-      | Some ((loc, _) as checksum) ->
-        let checksum = Checksum.of_string_user_error checksum |> User_error.ok_exn in
-        Some (loc, checksum)
-    in
-    { url; checksum }
-  ;;
-
-  let decode =
-    let open Decoder in
-    sum
-      [ ( Fields.copy
-        , located string
-          >>| fun (loc, source) path ->
-          External_copy
-            ( loc
-            , if Filename.is_relative source
-              then Path.External.relative path source
-              else Path.External.of_string source ) )
-      ; ( Fields.fetch
-        , let+ fetch = fields decode_fetch in
-          fun _ -> Fetch fetch )
-      ]
-  ;;
-
-  let encode_fetch_field { url = _loc, url; checksum } =
-    let open Encoder in
-    [ field Fields.url string url
-    ; field_o Fields.checksum Checksum.encode (Option.map checksum ~f:snd)
-    ]
-  ;;
-
-  let encode t =
-    let open Encoder in
-    match t with
-    | External_copy (_loc, path) ->
-      constr Fields.copy string (Path.External.to_string path)
-    | Fetch fetch -> named_record_fields Fields.fetch (encode_fetch_field fetch)
-  ;;
-end
-
 module Pkg_info = struct
   type t =
     { name : Package_name.t
@@ -146,15 +46,15 @@ module Pkg = struct
   type t =
     { build_command : Action.t option
     ; install_command : Action.t option
-    ; deps : (Loc.t * Package_name.t) list
+    ; depends : (Loc.t * Package_name.t) list
     ; info : Pkg_info.t
     ; exported_env : String_with_vars.t Action.Env_update.t list
     }
 
-  let equal { build_command; install_command; deps; info; exported_env } t =
+  let equal { build_command; install_command; depends; info; exported_env } t =
     Option.equal Action.equal_no_locs build_command t.build_command
     && Option.equal Action.equal_no_locs install_command t.install_command
-    && List.equal (Tuple.T2.equal Loc.equal Package_name.equal) deps t.deps
+    && List.equal (Tuple.T2.equal Loc.equal Package_name.equal) depends t.depends
     && Pkg_info.equal info t.info
     && List.equal
          (Action.Env_update.equal String_with_vars.equal)
@@ -162,31 +62,43 @@ module Pkg = struct
          t.exported_env
   ;;
 
-  let remove_locs t =
-    { t with
-      info = Pkg_info.remove_locs t.info
+  let remove_locs { build_command; install_command; depends; info; exported_env } =
+    { info = Pkg_info.remove_locs info
     ; exported_env =
-        List.map t.exported_env ~f:(Action.Env_update.map ~f:String_with_vars.remove_locs)
-    ; deps = List.map t.deps ~f:(fun (_, pkg) -> Loc.none, pkg)
+        List.map exported_env ~f:(Action.Env_update.map ~f:String_with_vars.remove_locs)
+    ; depends = List.map depends ~f:(fun (_, pkg) -> Loc.none, pkg)
+    ; build_command = Option.map build_command ~f:Action.remove_locs
+    ; install_command = Option.map install_command ~f:Action.remove_locs
     }
   ;;
 
-  let to_dyn { build_command; install_command; deps; info; exported_env } =
+  let to_dyn { build_command; install_command; depends; info; exported_env } =
     Dyn.record
       [ "build_command", Dyn.option Action.to_dyn build_command
       ; "install_command", Dyn.option Action.to_dyn install_command
-      ; "deps", Dyn.list (Dyn.pair Loc.to_dyn_hum Package_name.to_dyn) deps
+      ; "depends", Dyn.list (Dyn.pair Loc.to_dyn_hum Package_name.to_dyn) depends
       ; "info", Pkg_info.to_dyn info
       ; ( "exported_env"
         , Dyn.list (Action.Env_update.to_dyn String_with_vars.to_dyn) exported_env )
       ]
   ;;
 
+  let compute_missing_checksum t ~pinned =
+    let open Fiber.O in
+    let+ source =
+      match t.info.source with
+      | None -> Fiber.return None
+      | Some source ->
+        Source.compute_missing_checksum source t.info.name ~pinned >>| Option.some
+    in
+    { t with info = { t.info with source } }
+  ;;
+
   module Fields = struct
     let version = "version"
     let install = "install"
     let build = "build"
-    let deps = "deps"
+    let depends = "depends"
     let source = "source"
     let dev = "dev"
     let exported_env = "exported_env"
@@ -197,11 +109,11 @@ module Pkg = struct
     let open Decoder in
     enter
     @@ fields
-    @@ let+ version =
-         field ~default:Pkg_info.default_version Fields.version Package_version.decode
+    @@ let+ version = field Fields.version Package_version.decode
        and+ install_command = field_o Fields.install Action.decode_pkg
        and+ build_command = field_o Fields.build Action.decode_pkg
-       and+ deps = field ~default:[] Fields.deps (repeat (located Package_name.decode))
+       and+ depends =
+         field ~default:[] Fields.depends (repeat (located Package_name.decode))
        and+ source = field_o Fields.source Source.decode
        and+ dev = field_b Fields.dev
        and+ exported_env =
@@ -226,7 +138,7 @@ module Pkg = struct
            in
            { Pkg_info.name; version; dev; source; extra_sources }
          in
-         { build_command; deps; install_command; info; exported_env }
+         { build_command; depends; install_command; info; exported_env }
   ;;
 
   let encode_extra_source (local, source) : Dune_sexp.t =
@@ -239,7 +151,7 @@ module Pkg = struct
   let encode
     { build_command
     ; install_command
-    ; deps
+    ; depends
     ; info = { Pkg_info.name = _; extra_sources; version; dev; source }
     ; exported_env
     }
@@ -249,7 +161,7 @@ module Pkg = struct
       [ field Fields.version Package_version.encode version
       ; field_o Fields.install Action.encode install_command
       ; field_o Fields.build Action.encode build_command
-      ; field_l Fields.deps Package_name.encode (List.map deps ~f:snd)
+      ; field_l Fields.depends Package_name.encode (List.map depends ~f:snd)
       ; field_o Fields.source Source.encode source
       ; field_b Fields.dev dev
       ; field_l Fields.exported_env Action.Env_update.encode exported_env
@@ -367,7 +279,7 @@ let validate_packages packages =
   let missing_dependencies =
     Package_name.Map.values packages
     |> List.concat_map ~f:(fun (dependant_package : Pkg.t) ->
-      List.filter_map dependant_package.deps ~f:(fun (loc, dependency) ->
+      List.filter_map dependant_package.depends ~f:(fun (loc, dependency) ->
         if Package_name.Map.mem packages dependency
         then None
         else Some { dependant_package; dependency; loc }))
@@ -387,20 +299,17 @@ let create_latest_version
   (match validate_packages packages with
    | Ok () -> ()
    | Error (`Missing_dependencies missing_dependencies) ->
-     Code_error.raise
-       "Invalid package table"
-       (List.map
-          missing_dependencies
-          ~f:(fun { dependant_package; dependency; loc = _ } ->
-            ( "missing dependency"
-            , Dyn.record
-                [ "missing package", Package_name.to_dyn dependency
-                ; "dependency of", Package_name.to_dyn dependant_package.info.name
-                ] ))));
-  let version = Syntax.greatest_supported_version Dune_lang.Pkg.syntax in
+     List.map missing_dependencies ~f:(fun { dependant_package; dependency; loc = _ } ->
+       ( "missing dependency"
+       , Dyn.record
+           [ "missing package", Package_name.to_dyn dependency
+           ; "dependency of", Package_name.to_dyn dependant_package.info.name
+           ] ))
+     |> Code_error.raise "Invalid package table");
+  let version = Syntax.greatest_supported_version_exn Dune_lang.Pkg.syntax in
   let dependency_hash =
     Local_package.(
-      Dependency_set.hash (For_solver.list_non_local_dependency_set local_packages))
+      For_solver.list_non_local_dependency_set local_packages |> Dependency_set.hash)
     |> Option.map ~f:(fun dependency_hash -> Loc.none, dependency_hash)
   in
   let complete, used =
@@ -411,17 +320,13 @@ let create_latest_version
       let complete = Int.equal (List.length repos) (List.length used) in
       complete, Some used
   in
-  let repos : Repositories.t = { complete; used } in
-  let t =
-    { version
-    ; dependency_hash
-    ; packages
-    ; ocaml
-    ; repos
-    ; expanded_solver_variable_bindings
-    }
-  in
-  t
+  { version
+  ; dependency_hash
+  ; packages
+  ; ocaml
+  ; repos = { complete; used }
+  ; expanded_solver_variable_bindings
+  }
 ;;
 
 let default_path = Path.Source.(relative root "dune.lock")
@@ -497,7 +402,7 @@ module Package_filename = struct
 
   let to_package_name package_filename =
     if String.equal (Filename.extension package_filename) file_extension
-    then Ok (Filename.chop_extension package_filename |> Package_name.of_string)
+    then Ok (Filename.remove_extension package_filename |> Package_name.of_string)
     else Error `Bad_extension
   ;;
 end
@@ -530,7 +435,7 @@ module Write_disk = struct
     | Ok _ -> Error `Not_directory
   ;;
 
-  (* Removes the exitsing lock directory at the specified path if it exists and
+  (* Removes the existing lock directory at the specified path if it exists and
      is a valid lock directory. Checks the validity of the existing lockdir (if
      any) and raises if it's invalid before constructing the returned thunk, so
      validation can happen separately from executing the side effect that removes
@@ -548,12 +453,14 @@ module Write_disk = struct
           Pp.textf "Specified lock dir lacks metadata file (%s)" metadata_filename
         | `Failed_to_parse_metadata (path, exn) ->
           Pp.concat
-            ~sep:Pp.newline
+            ~sep:Pp.cut
             [ Pp.textf
                 "Unable to parse lock directory metadata file (%s):"
                 (Path.to_string_maybe_quoted path)
-            ; Exn.pp exn
+              |> Pp.hovbox
+            ; Exn.pp exn |> Pp.hovbox
             ]
+          |> Pp.vbox
       in
       User_error.raise
         [ Pp.textf
@@ -790,7 +697,7 @@ let transitive_dependency_closure t start =
              that its map of dependencies is closed under "depends on". *)
           Package_name.Set.(
             diff
-              (of_list_map (Package_name.Map.find_exn t.packages node).deps ~f:snd)
+              (of_list_map (Package_name.Map.find_exn t.packages node).depends ~f:snd)
               seen)
         in
         push_set unseen_deps;
@@ -798,3 +705,16 @@ let transitive_dependency_closure t start =
     in
     Ok (loop start)
 ;;
+
+let compute_missing_checksums t ~pinned_packages =
+  let open Fiber.O in
+  let+ packages =
+    Package_name.Map.to_list t.packages
+    |> Fiber.parallel_map ~f:(fun (name, pkg) ->
+      let pinned = Package_name.Set.mem pinned_packages name in
+      let+ pkg = Pkg.compute_missing_checksum pkg ~pinned in
+      name, pkg)
+    >>| Package_name.Map.of_list_exn
+  in
+  { t with packages }
+;;
